// Copyright 2024 schukai GmbH
// SPDX-License-Identifier: proprietary

package pathfinder

import (
	"github.com/stretchr/testify/assert"
	"testing"
)

// Test für SetValue
func TestSetValue(t *testing.T) {
	type Inner struct {
		Field PathValue
	}
	type Outer struct {
		InnerField *Inner
	}

	obj := &Outer{
		InnerField: &Inner{
			Field: "oldValue",
		},
	}

	v, err := GetValue[Outer](*obj, "InnerField.Field")
	if err != nil {
		t.Error(err)
	}

	assert.Equal(t, PathValue("oldValue"), v)

	nv := PathValue("newValue")
	err = SetValue[*Outer](obj, "InnerField.Field", nv)
	if err != nil {
		t.Fatalf("unexpected error: %v", err)
	}

	if obj.InnerField.Field != "newValue" {
		t.Fatalf("expected newValue, got: %s", obj.InnerField.Field)
	}

	v, err = GetValue[Outer](*obj, "InnerField.Field")
	if err != nil {
		t.Error(err)
	}

	assert.Equal(t, v, nv)

}

type Issue7Routing struct {
	P PathValue `json:"p" yaml:"p"`
	X string    `json:"x" yaml:"x"`
}

type Issue7Server struct {
	Routing []Issue7Routing `json:"routing" yaml:"routing"`
}

type Issue7Config struct {
	Server Issue7Server `json:"server" yaml:"server"`
}

func TestPathRewrite(t *testing.T) {

	obj := Issue7Config{
		Server: Issue7Server{
			Routing: []Issue7Routing{
				{
					P: "./test",
					X: "testX",
				},
			},
		},
	}

	v, err := GetValue[*Issue7Config](&obj, "Server.Routing.0.P")
	assert.Nil(t, err)

	assert.Equal(t, PathValue("./test"), v)

	nv := PathValue("newValue")
	err = SetValue[*Issue7Config](&obj, "Server.Routing.0.P", nv)
	assert.Nil(t, err)

	if obj.Server.Routing[0].P != "newValue" {
		t.Fatalf("expected newValue, got: %s", obj.Server.Routing[0].P)
	}

	v, err = GetValue[*Issue7Config](&obj, "Server.Routing.0.P")
	assert.Nil(t, err)
	assert.Equal(t, v, nv)

}

// Test data structs
type Issue7TestStruct2 struct {
	D map[string]PathValue
}

func TestPathRewrite2(t *testing.T) {
	// Test case 2
	ts2 := Issue7TestStruct2{
		D: map[string]PathValue{
			"key1": "yyy",
			"key2": "zzz",
		},
	}

	v, err := GetValue[Issue7TestStruct2](ts2, "D.key1")
	assert.Nil(t, err)
	assert.Equal(t, PathValue("yyy"), v)

	v, err = GetValue[Issue7TestStruct2](ts2, "D.key2")
	assert.Nil(t, err)
	assert.Equal(t, PathValue("zzz"), v)

	err = SetValue[*Issue7TestStruct2](&ts2, "D.key1", "xxx")
	assert.Nil(t, err)

	v, err = GetValue[Issue7TestStruct2](ts2, "D.key1")
	assert.Nil(t, err)
	assert.Equal(t, PathValue("xxx"), v)

	v, err = GetValue[Issue7TestStruct2](ts2, "D.key2")
	assert.Nil(t, err)
	assert.Equal(t, PathValue("zzz"), v)
}
